
#pragma ident "@(#)hello.c	1.3 95/05/24"

/*
 * DPS "Hello World" example.
 */

#include <stdio.h>
#include <Xm/DrawingA.h>
#include <DPS/dpsXclient.h>

static const char* FONT_NAME = "Helvetica";
static const char* STRING    = "Hello World";
static const float FONT_SIZE = 72.0;


DPSContext
initialize_dps(
	Widget w,
	float* string_width 
)
{
    Dimension width, height;
    float pswidth, psheight;
    float string_height;
    XSetWindowAttributes attr;
    DPSContext ctxt;


    XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);

    ctxt
	= XDPSCreateSimpleContext(XtDisplay(w), XtWindow(w),
			  DefaultGCOfScreen(XtScreen(w)),
			  0, height,
			  DPSDefaultTextBackstop,	/* send output to stdout */
			  DPSDefaultErrorProc,		/* send errors to stdout */
			  (DPSSpace) NULL		/* create a new space */
		  );

    /* libdps requires this when used with Xt */
    XDPSSetEventDelivery(XtDisplay(w), dps_event_pass_through);


    /* Keep drawing area centered so we can forget about handing resizes. */
    attr.bit_gravity = CenterGravity;
    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBitGravity, &attr);


    /* Set font and find out how wide the string is with this font. */
    DPSselectfont(ctxt, FONT_NAME, FONT_SIZE);
    DPSstringwidth(ctxt, STRING, string_width, &string_height);


    /* Move origin to center of drawing area.
     * This can be done faster, see the README file for details.
     */
    DPSitransform(ctxt, (float)width, (float)height, &pswidth, &psheight);
    DPStranslate(ctxt, pswidth/2.0, -psheight/2.0);

    return ctxt;
}


/* XmNexposeCallback on drawa widget */
static void
exposed(
	Widget w, 
	XtPointer client_data, 
	XmDrawingAreaCallbackStruct* cbs
)
{
    static DPSContext ctxt = NULL;
    static float string_width;

    if ( ctxt == NULL ) {
	ctxt = initialize_dps(w, &string_width);
    }

    DPSmoveto(ctxt, -(string_width/2.0), -(FONT_SIZE/2.0));
#ifdef FOR_FUN
    /* To turn on, un-#ifdef and uncomment wraps.psw in Makefile */
    PSWDrawShadowString(ctxt, STRING);
#endif
    DPSshow(ctxt, STRING);
    DPSFlushContext(ctxt);
}


void
main(
	int argc, 
	char **argv
)
{
    Widget topl, drawa;
    XtAppContext app;
    XEvent event;
    static const String fallbacks[] = {
	"hello.title: Hello World",
	"*.drawa.background: white",
	"*.drawa.width: 600",
	"*.drawa.height: 300",
	NULL
    };

    topl
	= XtVaAppInitialize(&app, "hello", NULL, 0, 
			    &argc, argv, 
			    (String*)fallbacks, 
			    NULL);
    drawa
	= XtVaCreateManagedWidget("drawa", xmDrawingAreaWidgetClass, topl,
				  NULL);
    XtAddCallback(drawa, XmNexposeCallback, (XtCallbackProc)exposed, NULL);

    XtRealizeWidget(topl);

    while ( True ) {
	/* DPS friendly equivalent of XtAppMainLoop() */
	XtAppNextEvent(app, &event);
	if ( !XDPSDispatchEvent(&event) )
	    (void) XtDispatchEvent(&event);
    }
}

