
#pragma ident "@(#)xvsps.c	1.5 95/05/24 SMI"

/*
 * A simple comparison of drawing in color with Xlib
 * versus Display PostScript.
 */

#include <stdio.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <DPS/dpsXclient.h>
#include <DPS/psops.h>

static const int   LINEHEIGHT = 24;
static const float PSLINEHEIGHT = 24.0;

static void
draw_lines_with_xlib(Widget w)
{
    Display* dpy  = XtDisplay(w);
    Window   win  = XtWindow(w);
    XGCValues gcvals;
    static XColor red, green, blue;
    static GC gc  = NULL;
    int yval;
    Dimension width;

    if ( gc == NULL ) {		/* first time through */
	Colormap cmap = DefaultColormapOfScreen(XtScreen(w));
	XColor unused;
	Status status;

	gcvals.line_width = LINEHEIGHT;
	gc = XCreateGC(dpy, win, GCLineWidth, &gcvals);

	status = XAllocNamedColor(dpy, cmap, "red", &red, &unused);
	if ( status == 0 ) {
	    fprintf(stderr, "XAllocNamedColor failed to allocate red cell\n");
	    return;
	}
	
	status = XAllocNamedColor(dpy, cmap, "green", &green, &unused);
	if ( status == 0 ) {
	    fprintf(stderr, "XAllocNamedColor failed to allocate green cell\n");
	    return;
	}

	status = XAllocNamedColor(dpy, cmap, "blue", &blue, &unused);
	if ( status == 0 ) {
	    fprintf(stderr, "XAllocNamedColor failed to allocate blue cell\n");
	    return;
	}
    }


    XtVaGetValues(w, XtNwidth, &width, NULL);

    /* Draw a red line */
    gcvals.foreground = red.pixel;
    XChangeGC(dpy, gc, GCForeground, &gcvals);
    yval = LINEHEIGHT/2;
    XDrawLine(dpy, win, gc, 0, yval, width, yval);

    /* Draw a green line */
    gcvals.foreground = green.pixel;
    XChangeGC(dpy, gc, GCForeground, &gcvals);
    yval += LINEHEIGHT;
    XDrawLine(dpy, win, gc, 0, yval, width, yval);

    /* Draw a blue line */
    gcvals.foreground = blue.pixel;
    XChangeGC(dpy, gc, GCForeground, &gcvals);
    yval += LINEHEIGHT;
    XDrawLine(dpy, win, gc, 0, yval, width, yval);
}


static void
draw_lines_with_dps(Widget w)
{
    static DPSContext ctxt = NULL;
    float yval;
    Dimension width;

    if ( ctxt == NULL ) {
	Dimension height;

	XtVaGetValues(w, XtNheight, &height, NULL);
	ctxt 
	    = XDPSCreateSimpleContext(XtDisplay(w), XtWindow(w),
				      DefaultGCOfScreen(XtScreen(w)),
				      0, height, 
				      DPSDefaultTextBackstop,
				      DPSDefaultErrorProc,
				      (DPSSpace) NULL
		);
	XDPSSetEventDelivery(XtDisplay(w), dps_event_pass_through);

	DPSSetContext(ctxt);	/* make this context the default */
	PSsetlinewidth(PSLINEHEIGHT);

#ifdef FOR_FUN
	/* It's easy to get PostScript to do something more interesting... */
	PSselectfont("Times-Roman", PSLINEHEIGHT*3.0);	/* pick a font */
	PSmoveto(PSLINEHEIGHT/2.0, PSLINEHEIGHT/1.25);	/* ad hoc centering */
	PScharpath("PostScript", True);			/* turn text into an outline */
	PSclip();					/* turn outline into clipping */
#endif
    }


    XtVaGetValues(w, XtNwidth, &width, NULL);

    /* Draw a red line */
    PSsetrgbcolor(1.0, 0.0, 0.0);
    yval = PSLINEHEIGHT/2.0;
    PSmoveto(0.0, yval);
    PSlineto((float)width, yval);
    PSstroke();

    /* Draw a green line */
    PSsetrgbcolor(0.0, 1.0, 0.0);
    yval += PSLINEHEIGHT;
    PSmoveto(0.0, yval);
    PSlineto((float)width, yval);
    PSstroke();

    /* Draw a blue line */
    PSsetrgbcolor(0.0, 0.0, 1.0);
    yval += PSLINEHEIGHT;
    PSmoveto(0.0, yval);
    PSlineto((float)width, yval);
    PSstroke();
}


static void
exposed(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct* cbs)
{
    draw_lines_with_dps( w );
    draw_lines_with_xlib( w );
}




void
main(int argc, char **argv)
{
    Widget topl;
    Widget form;
    Widget drawa;
    XtAppContext app;
    XEvent event;
    static const String fallbacks[] = {
	"xvsps.title: DPS vs. Xlib",
	"*.form.width: 400",
	"*.form.height: 200",
	"*.drawa.background: white",
	NULL
    };


    topl
	= XtVaAppInitialize(&app, "xvsps", NULL, 0, 
			    &argc, argv, (String *)fallbacks, 
			    NULL);

    form 
	= XtVaCreateManagedWidget("form", xmFormWidgetClass, topl,
				  NULL);

    drawa
	= XtVaCreateManagedWidget("drawa", xmDrawingAreaWidgetClass, form,
				  XmNtopAttachment,	XmATTACH_FORM,
				  XmNrightAttachment,	XmATTACH_FORM,
				  XmNbottomAttachment,	XmATTACH_FORM,
				  XmNleftAttachment,	XmATTACH_FORM,
				  NULL);
    XtAddCallback(drawa, XmNexposeCallback, (XtCallbackProc)exposed, NULL);


    XtRealizeWidget(topl);

    while ( True ) {
	XtAppNextEvent(app, &event);
	if ( !XDPSDispatchEvent(&event) )
	    (void) XtDispatchEvent(&event);
    }
}
/******************************************************************************/
